%% T-61.2010 Datasta tietoon / T#2 / 1
% Jukka Parviainen, 14.11.2006

%% Principal Component Analysis
% Let's take an example from round #2 of the "paper" exercises 
% problem 1

%% Data
% The 4 points of the exercise problem in the 2D plane.

X = [2 5 6 7; ...
    1 3 5 7];
disp('The original data matrix X:');
disp(X);

%% Plot the points in (x1, x2)-plane

figure(11); clf;
p1 = plot(X(1,:), X(2,:), 'bo');
set(p1, 'MarkerFaceColor',[0 0 1]);
grid on;
axis equal
set(gca,'XTick',[-2:10]);
set(gca,'YTick',[-2:10]);
axis([-1 9 -1 9]);
title('Data points');

for k = [1 : size(X, 2)]
    text(X(1,k),X(2,k),['(' num2str(X(1,k)) ',' num2str(X(2,k)) ')']);
end;

%% Averaging the data

ka = (mean(X,2));
XK = X - repmat(ka, 1, size(X,2));

disp('The mean value ka:');
disp(ka);
disp('that is removed, which results in XK:');
disp(XK);

figure(11); hold on;
XC = ka(1)+[-3 0; 3 0];
YC = ka(2)+[0 -3; 0 3];
lin8 = line(XC, YC);
set(lin8,'Color',[0 1 0]);
title(['Averaged data points, ka = (' num2str(ka(1)) ',' num2str(ka(2)) ')']);

for k = [1 : size(X, 2)]
    t1 = text(X(1,k),X(2,k)-0.5,['(' num2str(XK(1,k)) ',' num2str(XK(2,k)) ')']);
    set(t1,'Color',[0 1 0]);
end;

%% Find the principal components
% The goal is to find the direction along which
% the data has most variance


%% Form the covariance matrix
% The size of the covariance matrix will be (d x d), 
% in this case (2 x 2).

C = (1/4)*XK*XK';
disp('Covariance matrix C:');
disp(C);


%% Find the eigenvalues
% Eigenvalues \lambda and eigenvectors x of a square matrix A are
% defined by the equation
%$$A x = \lambda x$$
%
% in Matlab : type "help eig"

[V, D] = eig(C);

disp('eigenvectors of C in columns:');
disp(V);

disp('eigenvalues of C on the diagonal:');
disp(D);

%% Sort the eigenvectors according to descending eigenvalues
% After this step we know that in the first column of V we have
% the eigenvector that corresponds to the largest of the eigenvalues.
% Also the other eigenvectors are in the order of descending eigenvalues.

[dummy1, ind] = sort(diag(D));
ind = flipud(ind);
V = V(:, ind);
D = D(:, ind);


%% Eigenvalues
% The cumulative sum lambdacs of the eigenvalues is found.
%
% This way we know that the first m eigenvectors 
% explain the fraction labdacs(m)/lambdacs(d) of
% the variance of the whole data. This can be used to reduce the data 
% dimensionality without introducing too large an error 
% as usually the additional variance contained in the 
% "last" dimensions is small

lambda = sum(D);
lambdacs = cumsum(lambda);
lambdasum = sum(lambda);

disp('Eigenvalue #k | Cumulative sum of the eigenvalues %');
disp([ [1:length(lambda)]'  100*lambdacs'/lambdasum]);

%% Project the 2D data into 1D space (line)
% Use the "largest" principal component as projection matrix

disp('PCA1-vector:');
disp(V(:,1))
disp('Percentage of data variance explained (%):');
disp(100*sum(D(:,1))/sum(D(:)));

%P  = V(:,1)' * X
PK = V(:,1)' * XK;
disp('Values of the 1D projections:');
disp(PK);

%% Lets also plot the projected points in the 2D space (plane)
% They end up on a straight line that is determined by the 
% largest principal component
%
% If thought as coding/decoding, we have to store
% four 1D points (4 data units) and the necessary eigenvector(s)
% (2 units) instead of the original 
% four 2D points (8 units).

XP = PK * V(1,1) + ka(1);
YP = PK * V(2,1) + ka(2);
p2 = plot(XP, YP, 'ro', 'MarkerSize', 12);

for k = [1 : size(XP, 2)]
    t1 = text(XP(k),YP(k),['(' num2str(XP(k)) ',' num2str(YP(k)) ')']);
    set(t1,'Color',[1 0 0]);
end;

p3 = line([-V(1,1) V(1,1)]*10+ka(1), [-V(2,1) V(2,1)]*10+ka(2));
set(p3, 'LineStyle', '-.', 'Color', [1 0 0]);

disp('Projected points in 2D');
disp([XP; YP]);



