%% T-61.2010 Datasta tietoon / T#2 / 2
% Jukka Parviainen, 14.11.2006
%
% Load the data matrix, look at it and
% perform PCA (Principal Component Analysis).

clc;        % clear the screen
clear       % clear all variables from the workspace
close all   % close all windows

%% Load the file
load X.txt -ascii

%% Check what variables we got into the matlab workspace
whos

%% What is the dimensionality of the variable X?
% In Matlab the number of rows is reported first,
% then the number of columns (and after that the sizes of the 
% remaining dimensions if there are any).
%
% Here we assume the the dimension of the data grown downwards,
% and the number of data points to the right

size(X)
disp('Number of data points:');
disp(size(X,2));
disp('Dimension of the data vector:');
disp(size(X,1));

%% Print all the numbers to the screen

X

%% Draw a 3D plot of the data
% After drawing rotate the plot
% (with command "rotate3d").

figure(21); clf;
plot3(X(1,:), X(2,:), X(3,:), 'x');
grid on;
title(['3D-data, N = 140']);
rotate3d

%% Something is not right???
% Examine the mean values, minimum and maximum values and 
% the histograms of the data components

mean(X')
min(X')
max(X')

figure(22); clf;
hist(X');
title('Component-wise histogram of the data');
xlabel('Value');
ylabel('Number of hits');
legend({'#1','#2','#3'});

%% Data preprocessing
% Usually in real-life projects the data pre-processing is the
% the most time consuming part. It affects very significantly
% the quality of the final results obtained.
%

% It was observed that the value of one data sample is 
% exceptionally large - this might be due to an erraneous measurement,
% an error when recording / retrieving the data or some other reason
%
% A simple preprocessing step is now applied - the erraneous value ("outlier")
% is simply removed from the data set. Other alternatives:
% replacing by mean vule, value given by regression function,...

ind = find(X(2,:) > -100);
X   = X(:,ind);
size(X)

%% Redraw
% Redo the rotating

figure(21); clf;
plot3(X(1,:), X(2,:), X(3,:), 'x');
grid on;
title(['Pre-processed 3D data , N = 139']);
rotate3d

%% Tasks
%
% * Rotate the axis - you of course see only a projection
% in 2D plane at time but by rotating you can grap the feeling of the three
% dimensions
%   * how many clusters you think there are?
%   * can you rotate so that there seems to be only two clusters
%   * ... only one cluster?
% * Leave to the position where the clusters are most clearly separated

%% Perform PCA
% The PCA calculation is detailed in the paper exercise round #2 problem 1.
% First the mean value is subtracted, then the covariance matrix of the data
% is calculated, then the eigenvectors and eigenvalues of the covariance matrix
% 

N  = size(X,2);
ka = mean(X,2);
XK = X - repmat(ka, 1, N);
C  = (1/N)*XK*XK';  
[V,D] = eig(C);
kokon = sum(D(:));

%% Ensure that the eigenvector corresponding to the largest eigenvalue comes 
% first 
[dummy,jarj] = sort(diag(D));
jarj = flipud(jarj);
V = V(:,jarj);
D = D(:,jarj);

%% Determine the percentages
pross = diag(D)/kokon;
saily = 100*(pross(1)+pross(2));
disp(['Retain ' num2str(saily) ' % of the variation']);

%% Project to 2D plane

Y = V(:,1:2)' * X;

figure(23); clf;
plot(Y(1,:), Y(2,:), 'x');
grid on;
axis equal;
title(['2D projection of 3D data, retained  ' num2str(saily) '%']);

P = V(:,1:2) * Y;
figure(21); hold on;
plot3(P(1,:),P(2,:),P(3,:),'ro');
title(['3D data with pre-processing + projection to 2D, N = 139']);
rotate3d

