%% T-61.2010 Datasta tietoon / T#2 / 3
% Jukka Parviainen, 14.11.2006

clc;
clear;
close all;

%% Eigenfaces
% Chapter 4 of the lectures, PCA: eigenfaces
%
% Data consists of facial images that are read as a matrix X.
% A column of this matrix then contains one image scanned pixel by pixel.
% For example, from with 10 images with size 50x50 pixels,
% we would get 2500 rows and 10 columns.

%% Image collection
% We use MIT's CBCL image database. See
% |http://cbcl.mit.edu/cbcl/software-datasets/FaceData2.html|
%
% Each facial image is 19 x 19 pixels, this results
% in datavector dimension 361. 
%
% NOTICE! You may also choose any other data file from the directory
% |http://www.cis.hut.fi/Opinnot/T-61.2010/Harjoitustyo/|
% from section "Topic #1". Then you have to do 
% the transformations to the data (the cell below)
% yourself.

load dt_t2_m3_K.mat
% contains variable K

%% Draw the facial images
% Use the helper function myMontage.m and
% draw the images to window 31.

myMontage(K, 'Kasvokuvat', 31);

%% Convert to data matrix
% Here the elements of the image matrix are converted to a real data
% matrix X and forced to be of the data type "double".
% See e.g. help 
% reshape, help double.
%
% To save time and trouble, read a readily formed data matrix
% X from a file. Additionally the mean brightness value of an image 
% is subtracted from each of its pixels. This way we try to compensate for
% variation in imaging conditions.
%
% NOTE! If you decided to use some other data file,
% you should convert the image data to matrix X

load dt_t2_m3_X.mat
% contains variable X  (361 rows==pixels, N columns==images)

%% Make sure the sizes of the matrices are
% K: image matrix  with dimensions (r x c x 1 x n)
% X: data matrix (rc x n)
%
% Thus the following numbers of elements should be equal

disp(['Elements in K: ' num2str(prod(size(K))) ', in X: ' num2str(prod(size(X)))]);

%% Perform PCA on the data
% Remove mean, calculate covariance matrix, find its
% eigenvalues and eigenvectors

% WRITE YOUR CODE HERE! (See e.g. demo 2)

%% Project to 2D
% Project the face images to 2D (matrix Y).

Y = []; % insert the calculation of the projection here!!!

figure(32); clf;
for k=[1:size(Y,2)]
    plot(Y(1,k), Y(2,k), 'bx');
    hold on;
    ti = text(Y(1,k),Y(2,k),[num2str(k)]);
    set(ti,'Color',[0 0 1]);
end;
grid on;
title(['Projection of the facial images to 2D']);

%% Load test data. faces and non-faces
% Read two new test sets containing faces and non-faces
% Project to 2D using the same projection matrix.

load dt_t2_m3_K2.mat
myMontage(K2, 'Test set FACES', 33);
load dt_t2_m3_K3.mat
myMontage(K3, 'Test set NON-FACES', 34);

% In the following the readily preprocessed data matrices:
load dt_t2_m3_X2.mat
% contains the variable X2  (361 rows==pixels, N columns==images)
load dt_t2_m3_X3.mat
% contains the variable X3  (361 rows==pixels, N columns==images)

%% Projection with the projection matrix calculated earlier
% Calculate here the projections Y2 and Y3 using projection matrix 
% constructed from the two eigenvectors corresponding to largest eigenvalues
% (calculated  from the original data) 


Y2 = []; % insert the calculation of the projection here!!!
Y3 = []; % insert the calculation of the projection here!!!

figure(32);
hold on;
for k=[1:size(Y2,2)]
    plot(Y2(1,k), Y2(2,k), 'ro');
    ti = text(Y2(1,k)-3,Y2(2,k),[num2str(k)]);
    set(ti,'Color',[1 0 0]);
end;
for k=[1:size(Y3,2)]
    plot(Y3(1,k), Y3(2,k), 'kd');
    ti = text(Y3(1,k),Y3(2,k)+3,[num2str(k)]);
    set(ti,'Color',[0 0 0]);
end;
title('Projektions: x face images train, o face images test, <> non-faces');
